=== modified file 'drizzled/optimizer/explain_plan.cc'
--- drizzled/optimizer/explain_plan.cc	2009-12-23 21:16:15 +0000
+++ drizzled/optimizer/explain_plan.cc	2010-01-27 04:37:14 +0000
@@ -52,6 +52,20 @@
   "index_merge"
 };
 
+static const string select_type_str[]=
+{
+  "PRIMARY",
+  "SIMPLE",
+  "DERIVED",
+  "DEPENDENT SUBQUERY",
+  "UNCACHEABLE SUBQUERY",
+  "SUBQUERY",
+  "DEPENDENT UNION",
+  "UNCACHEABLE_UNION",
+  "UNION",
+  "UNION RESULT"
+};
+
 void optimizer::ExplainPlan::printPlan()
 {
   List<Item> field_list;
@@ -73,8 +87,8 @@
   {
     item_list.push_back(new Item_int((int32_t)
                         join->select_lex->select_number));
-    item_list.push_back(new Item_string(join->select_lex->type.c_str(),
-                                        join->select_lex->type.length(),
+    item_list.push_back(new Item_string(select_type_str[join->select_lex->type].c_str(),
+                                        select_type_str[join->select_lex->type].length(),
                                         cs));
     for (uint32_t i= 0; i < 7; i++)
       item_list.push_back(item_null);
@@ -100,8 +114,8 @@
     /* id */
     item_list.push_back(new Item_null);
     /* select_type */
-    item_list.push_back(new Item_string(join->select_lex->type.c_str(),
-                                        join->select_lex->type.length(),
+    item_list.push_back(new Item_string(select_type_str[join->select_lex->type].c_str(),
+                                        select_type_str[join->select_lex->type].length(),
                                         cs));
     /* table */
     {
@@ -180,8 +194,8 @@
       item_list.push_back(new Item_uint((uint32_t)
             join->select_lex->select_number));
       /* select_type */
-      item_list.push_back(new Item_string(join->select_lex->type.c_str(),
-                                          join->select_lex->type.length(),
+      item_list.push_back(new Item_string(select_type_str[join->select_lex->type].c_str(),
+                                          select_type_str[join->select_lex->type].length(),
                                           cs));
       if (tab->type == AM_ALL && tab->select && tab->select->quick)
       {
@@ -473,11 +487,11 @@
     {
       if (sl->first_inner_unit() || sl->next_select())
       {
-        sl->type.assign("PRIMARY");
+        sl->type= optimizer::ST_PRIMARY;
       }
       else
       {
-        sl->type.assign("SIMPLE");
+        sl->type= optimizer::ST_SIMPLE;
       }
     }
     else
@@ -486,23 +500,23 @@
       {
         if (sl->linkage == DERIVED_TABLE_TYPE)
         {
-          sl->type.assign("DERIVED");
+          sl->type= optimizer::ST_DERIVED;
         }
         else
         {
           if (uncacheable & UNCACHEABLE_DEPENDENT)
           {
-            sl->type.assign("DEPENDENT SUBQUERY");
+            sl->type= optimizer::ST_DEPENDENT_SUBQUERY;
           }
           else
           {
             if (uncacheable)
             {
-              sl->type.assign("UNCACHEABLE SUBQUERY");
+              sl->type= optimizer::ST_UNCACHEABLE_SUBQUERY;
             }
             else
             {
-              sl->type.assign("SUBQUERY");
+              sl->type= optimizer::ST_SUBQUERY;
             }
           }
         }
@@ -511,17 +525,17 @@
       {
         if (uncacheable & UNCACHEABLE_DEPENDENT)
         {
-          sl->type.assign("DEPENDENT UNION");
+          sl->type= optimizer::ST_DEPENDENT_UNION;
         }
         else
         {
           if (uncacheable)
           {
-            sl->type.assign("UNCACHEABLE_UNION");
+            sl->type= optimizer::ST_UNCACHEABLE_UNION;
           }
           else
           {
-            sl->type.assign("UNION");
+            sl->type= optimizer::ST_UNION;
           }
         }
       }
@@ -532,7 +546,7 @@
   if (unit->is_union())
   {
     unit->fake_select_lex->select_number= UINT_MAX; // just for initialization
-    unit->fake_select_lex->type.assign("UNION RESULT");
+    unit->fake_select_lex->type= optimizer::ST_UNION_RESULT;
     unit->fake_select_lex->options|= SELECT_DESCRIBE;
     if (! (res= unit->prepare(session, result, SELECT_NO_UNLOCK | SELECT_DESCRIBE)))
     {

=== modified file 'drizzled/optimizer/explain_plan.h'
--- drizzled/optimizer/explain_plan.h	2009-12-08 23:18:36 +0000
+++ drizzled/optimizer/explain_plan.h	2010-01-27 04:37:14 +0000
@@ -30,6 +30,21 @@
 namespace optimizer
 {
 
+/** select type for EXPLAIN */
+enum select_type
+{ 
+  ST_PRIMARY,
+  ST_SIMPLE,
+  ST_DERIVED,
+  ST_DEPENDENT_SUBQUERY,
+  ST_UNCACHEABLE_SUBQUERY,
+  ST_SUBQUERY,
+	ST_DEPENDENT_UNION,
+  ST_UNCACHEABLE_UNION,
+  ST_UNION,
+  ST_UNION_RESULT
+};
+
 class ExplainPlan
 {
 public:

=== modified file 'drizzled/optimizer/quick_group_min_max_select.cc'
--- drizzled/optimizer/quick_group_min_max_select.cc	2009-12-27 00:49:03 +0000
+++ drizzled/optimizer/quick_group_min_max_select.cc	2010-01-27 17:32:16 +0000
@@ -27,6 +27,9 @@
 #include "drizzled/optimizer/quick_range_select.h"
 #include "drizzled/optimizer/sel_arg.h"
 #include "drizzled/internal/m_string.h"
+#include "drizzled/util/functors.h"
+
+#include <vector>
 
 using namespace std;
 using namespace drizzled;
@@ -120,9 +123,6 @@
 
   if (min_max_arg_part)
   {
-    if (my_init_dynamic_array(&min_max_ranges, sizeof(optimizer::QuickRange*), 16, 16))
-      return 1;
-
     if (have_min)
     {
       if (! (min_functions= new List<Item_sum>))
@@ -160,8 +160,6 @@
         return 1;
     }
   }
-  else
-    min_max_ranges.elements= 0;
 
   return 0;
 }
@@ -175,8 +173,11 @@
   }
   if (min_max_arg_part)
   {
-    delete_dynamic(&min_max_ranges);
+    for_each(min_max_ranges.begin(),
+             min_max_ranges.end(),
+             DeletePtr());
   }
+  min_max_ranges.clear();
   free_root(&alloc,MYF(0));
   delete min_functions_it;
   delete max_functions_it;
@@ -212,8 +213,7 @@
                                    range_flag);
   if (! range)
     return true;
-  if (insert_dynamic(&min_max_ranges, (unsigned char*)&range))
-    return true;
+  min_max_ranges.push_back(range);
   return false;
 }
 
@@ -240,14 +240,12 @@
 void optimizer::QuickGroupMinMaxSelect::update_key_stat()
 {
   max_used_key_length= real_prefix_len;
-  if (min_max_ranges.elements > 0)
+  if (! min_max_ranges.empty())
   {
     optimizer::QuickRange *cur_range= NULL;
     if (have_min)
     { /* Check if the right-most range has a lower boundary. */
-      get_dynamic(&min_max_ranges,
-                  (unsigned char*) &cur_range,
-                  min_max_ranges.elements - 1);
+      cur_range= min_max_ranges.back();
       if (! (cur_range->flag & NO_MIN_RANGE))
       {
         max_used_key_length+= min_max_arg_len;
@@ -257,7 +255,7 @@
     }
     if (have_max)
     { /* Check if the left-most range has an upper boundary. */
-      get_dynamic(&min_max_ranges, (unsigned char*)&cur_range, 0);
+      cur_range= min_max_ranges.front();
       if (! (cur_range->flag & NO_MAX_RANGE))
       {
         max_used_key_length+= min_max_arg_len;
@@ -387,7 +385,7 @@
   int result= 0;
 
   /* Find the MIN key using the eventually extended group prefix. */
-  if (min_max_ranges.elements > 0)
+  if (! min_max_ranges.empty())
   {
     if ((result= next_min_in_range()))
       return result;
@@ -452,7 +450,7 @@
   int result= 0;
 
   /* Get the last key in the (possibly extended) group. */
-  if (min_max_ranges.elements > 0)
+  if (! min_max_ranges.empty())
     result= next_max_in_range();
   else
     result= cursor->index_read_map(record,
@@ -520,17 +518,20 @@
 
   max_key.reserve(real_prefix_len + min_max_arg_len);
 
-  assert(min_max_ranges.elements > 0);
+  assert(! min_max_ranges.empty());
 
-  for (uint32_t range_idx= 0; range_idx < min_max_ranges.elements; range_idx++)
+  for (vector<optimizer::QuickRange *>::iterator it= min_max_ranges.begin();
+       it != min_max_ranges.end();
+       ++it)
   { /* Search from the left-most range to the right. */
-    get_dynamic(&min_max_ranges, (unsigned char*)&cur_range, range_idx);
+    cur_range= *it;
 
     /*
       If the current value for the min/max argument is bigger than the right
       boundary of cur_range, there is no need to check this range.
     */
-    if (range_idx != 0 && !(cur_range->flag & NO_MAX_RANGE) &&
+    if (it != min_max_ranges.begin() && 
+        ! (cur_range->flag & NO_MAX_RANGE) &&
         (key_cmp(min_max_arg_part,
                  (const unsigned char*) cur_range->max_key,
                  min_max_arg_len) == 1))
@@ -635,17 +636,19 @@
   basic_string<unsigned char> min_key;
   min_key.reserve(real_prefix_len + min_max_arg_len);
 
-  assert(min_max_ranges.elements > 0);
+  assert(! min_max_ranges.empty());
 
-  for (uint32_t range_idx= min_max_ranges.elements; range_idx > 0; range_idx--)
+  for (vector<optimizer::QuickRange *>::reverse_iterator rit= min_max_ranges.rbegin();
+       rit != min_max_ranges.rend();
+       ++rit)
   { /* Search from the right-most range to the left. */
-    get_dynamic(&min_max_ranges, (unsigned char*)&cur_range, range_idx - 1);
+    cur_range= *rit;
 
     /*
       If the current value for the min/max argument is smaller than the left
       boundary of cur_range, there is no need to check this range.
     */
-    if (range_idx != min_max_ranges.elements &&
+    if (rit != min_max_ranges.rbegin() &&
         ! (cur_range->flag & NO_MIN_RANGE) &&
         (key_cmp(min_max_arg_part,
                  (const unsigned char*) cur_range->min_key,

=== modified file 'drizzled/optimizer/quick_group_min_max_select.h'
--- drizzled/optimizer/quick_group_min_max_select.h	2009-12-25 08:50:15 +0000
+++ drizzled/optimizer/quick_group_min_max_select.h	2010-01-26 19:35:32 +0000
@@ -22,6 +22,8 @@
 
 #include "drizzled/optimizer/range.h"
 
+#include <vector>
+
 namespace drizzled
 {
 
@@ -81,7 +83,7 @@
   uint32_t min_max_arg_len; /**< The length of the MIN/MAX argument field */
   unsigned char *key_infix; /**< Infix of constants from equality predicates. */
   uint32_t key_infix_len;
-  DYNAMIC_ARRAY min_max_ranges; /**< Array of range ptrs for the MIN/MAX field. */
+  std::vector<QuickRange *> min_max_ranges; /**< Array of range ptrs for the MIN/MAX field. */
   uint32_t real_prefix_len; /**< Length of key prefix extended with key_infix. */
   uint32_t real_key_parts;  /**< A number of keyparts in the above value.      */
   List<Item_sum> *min_functions;
@@ -93,7 +95,7 @@
 
   /*
     The following two members are public to allow easy access from
-    TRP_GROUP_MIN_MAX::make_quick()
+    GroupMinMaxReadPlan::make_quick()
   */
   drizzled::memory::Root alloc; /**< Memory pool for this and quick_prefix_select data. */
   QuickRangeSelect *quick_prefix_select; /**< For retrieval of group prefixes. */

=== modified file 'drizzled/optimizer/quick_range_select.cc'
--- drizzled/optimizer/quick_range_select.cc	2009-12-27 00:49:03 +0000
+++ drizzled/optimizer/quick_range_select.cc	2010-01-26 18:41:37 +0000
@@ -457,16 +457,15 @@
 
 int optimizer::QuickRangeSelect::cmp_prev(optimizer::QuickRange *range_arg)
 {
-  int cmp;
   if (range_arg->flag & NO_MIN_RANGE)
-    return 0;					/* key can't be to small */
+    return 0; /* key can't be to small */
 
-  cmp= key_cmp(key_part_info,
-               range_arg->min_key,
-               range_arg->min_length);
+  int cmp= key_cmp(key_part_info,
+                   range_arg->min_key,
+                   range_arg->min_length);
   if (cmp > 0 || (cmp == 0 && (range_arg->flag & NEAR_MIN) == false))
     return 0;
-  return 1;                                     // outside of range
+  return 1; // outside of range
 }
 
 
@@ -500,25 +499,29 @@
  */
 optimizer::QuickSelectDescending::QuickSelectDescending(optimizer::QuickRangeSelect *q, uint32_t, bool *)
   :
-    optimizer::QuickRangeSelect(*q),
-    rev_it(rev_ranges)
+    optimizer::QuickRangeSelect(*q)
 {
-  optimizer::QuickRange *r= NULL;
-
   optimizer::QuickRange **pr= (optimizer::QuickRange**) ranges.buffer;
   optimizer::QuickRange **end_range= pr + ranges.elements;
   for (; pr != end_range; pr++)
-    rev_ranges.push_front(*pr);
+  {
+    rev_ranges.push_back(*pr);
+  }
+  rev_it= rev_ranges.begin();
 
   /* Remove EQ_RANGE flag for keys that are not using the full key */
-  for (r = rev_it++; r; r= rev_it++)
+  for (vector<optimizer::QuickRange *>::iterator it= rev_ranges.begin();
+       it != rev_ranges.end();
+       ++it)
   {
+    optimizer::QuickRange *r= *it;
     if ((r->flag & EQ_RANGE) &&
         head->key_info[index].key_length != r->max_length)
+    {
       r->flag&= ~EQ_RANGE;
+    }
   }
-  rev_it.rewind();
-  q->dont_free= 1;				// Don't free shared mem
+  q->dont_free= 1; // Don't free shared mem
   delete q;
 }
 
@@ -546,15 +549,19 @@
 		           cursor->index_prev(record));
       if (! result)
       {
-        if (cmp_prev(*rev_it.ref()) == 0)
-          return 0;
+          if (cmp_prev(*(rev_it - 1)) == 0)
+            return 0;
       }
       else if (result != HA_ERR_END_OF_FILE)
         return result;
     }
 
-    if (! (last_range= rev_it++))
-      return HA_ERR_END_OF_FILE;		// All ranges used
+    if (rev_it == rev_ranges.end())
+    {
+      return HA_ERR_END_OF_FILE; // All ranges used
+    }
+    last_range= *rev_it;
+    ++rev_it;
 
     if (last_range->flag & NO_MAX_RANGE)        // Read last record
     {

=== modified file 'drizzled/optimizer/quick_range_select.h'
--- drizzled/optimizer/quick_range_select.h	2009-12-25 08:50:15 +0000
+++ drizzled/optimizer/quick_range_select.h	2010-01-26 19:00:26 +0000
@@ -22,6 +22,8 @@
 
 #include "drizzled/optimizer/range.h"
 
+#include <vector>
+
 class Cursor;
 
 namespace drizzled
@@ -226,7 +228,7 @@
     mrr_buf_size= 0;
   }
 
-  friend class ::TRP_ROR_INTERSECT; 
+  friend class ::RorIntersectReadPlan; 
 
   friend
   QuickRangeSelect *get_quick_select_for_ref(Session *session, Table *table,
@@ -295,13 +297,13 @@
 
   int reset(void) 
   { 
-    rev_it.rewind(); 
+    rev_it= rev_ranges.begin();
     return QuickRangeSelect::reset();
   }
 
-  List<QuickRange> rev_ranges;
+  std::vector<QuickRange *> rev_ranges;
 
-  List_iterator<QuickRange> rev_it;
+  std::vector<QuickRange *>::iterator rev_it;
 
 };
 

=== modified file 'drizzled/optimizer/range.cc'
--- drizzled/optimizer/range.cc	2009-12-30 18:44:00 +0000
+++ drizzled/optimizer/range.cc	2010-01-26 19:12:51 +0000
@@ -313,30 +313,30 @@
                                   uint32_t *bufsize,
                                   COST_VECT *cost);
 
-static optimizer::TRP_RANGE *get_key_scans_params(optimizer::Parameter *param,
-                                                  SEL_TREE *tree,
-                                                  bool index_read_must_be_used,
-                                                  bool update_tbl_stats,
+static optimizer::RangeReadPlan *get_key_scans_params(optimizer::Parameter *param,
+                                                      SEL_TREE *tree,
+                                                      bool index_read_must_be_used,
+                                                      bool update_tbl_stats,
+                                                      double read_time);
+
+static
+optimizer::RorIntersectReadPlan *get_best_ror_intersect(const optimizer::Parameter *param,
+                                                        SEL_TREE *tree,
+                                                        double read_time,
+                                                        bool *are_all_covering);
+
+static
+optimizer::RorIntersectReadPlan *get_best_covering_ror_intersect(optimizer::Parameter *param,
+                                                                 SEL_TREE *tree,
+                                                                 double read_time);
+
+static
+optimizer::TableReadPlan *get_best_disjunct_quick(optimizer::Parameter *param,
+                                                  SEL_IMERGE *imerge,
                                                   double read_time);
 
 static
-optimizer::TRP_ROR_INTERSECT *get_best_ror_intersect(const optimizer::Parameter *param,
-                                                     SEL_TREE *tree,
-                                                     double read_time,
-                                                     bool *are_all_covering);
-
-static
-optimizer::TRP_ROR_INTERSECT *get_best_covering_ror_intersect(optimizer::Parameter *param,
-                                                              SEL_TREE *tree,
-                                                              double read_time);
-
-static
-optimizer::TABLE_READ_PLAN *get_best_disjunct_quick(optimizer::Parameter *param,
-                                                    SEL_IMERGE *imerge,
-                                                    double read_time);
-
-static
-optimizer::TRP_GROUP_MIN_MAX *get_best_group_min_max(optimizer::Parameter *param, SEL_TREE *tree);
+optimizer::GroupMinMaxReadPlan *get_best_group_min_max(optimizer::Parameter *param, SEL_TREE *tree);
 
 static void print_sel_tree(optimizer::Parameter *param,
                            SEL_TREE *tree,
@@ -1004,8 +1004,8 @@
         read_time= key_read_time;
     }
 
-    optimizer::TABLE_READ_PLAN *best_trp= NULL;
-    optimizer::TRP_GROUP_MIN_MAX *group_trp= NULL;
+    optimizer::TableReadPlan *best_trp= NULL;
+    optimizer::GroupMinMaxReadPlan *group_trp= NULL;
     double best_read_time= read_time;
 
     if (cond)
@@ -1051,8 +1051,8 @@
       */
       if (tree->merges.is_empty())
       {
-        optimizer::TRP_RANGE *range_trp= NULL;
-        optimizer::TRP_ROR_INTERSECT *rori_trp= NULL;
+        optimizer::RangeReadPlan *range_trp= NULL;
+        optimizer::RorIntersectReadPlan *rori_trp= NULL;
         bool can_build_covering= false;
 
         /* Get best 'range' plan and prepare data for making other plans */
@@ -1093,9 +1093,9 @@
       else
       {
         /* Try creating index_merge/ROR-union scan. */
-        SEL_IMERGE *imerge;
-        optimizer::TABLE_READ_PLAN *best_conj_trp= NULL;
-        optimizer::TABLE_READ_PLAN *new_conj_trp= NULL;
+        SEL_IMERGE *imerge= NULL;
+        optimizer::TableReadPlan *best_conj_trp= NULL;
+        optimizer::TableReadPlan *new_conj_trp= NULL;
         List_iterator_fast<SEL_IMERGE> it(tree->merges);
         while ((imerge= it++))
         {
@@ -1204,16 +1204,16 @@
 */
 
 static
-optimizer::TABLE_READ_PLAN *get_best_disjunct_quick(optimizer::Parameter *param,
-                                                    SEL_IMERGE *imerge,
-                                                    double read_time)
+optimizer::TableReadPlan *get_best_disjunct_quick(optimizer::Parameter *param,
+                                                  SEL_IMERGE *imerge,
+                                                  double read_time)
 {
   SEL_TREE **ptree;
-  optimizer::TRP_INDEX_MERGE *imerge_trp= NULL;
+  optimizer::IndexMergeReadPlan *imerge_trp= NULL;
   uint32_t n_child_scans= imerge->trees_next - imerge->trees;
-  optimizer::TRP_RANGE **range_scans= NULL;
-  optimizer::TRP_RANGE **cur_child= NULL;
-  optimizer::TRP_RANGE **cpk_scan= NULL;
+  optimizer::RangeReadPlan **range_scans= NULL;
+  optimizer::RangeReadPlan **cur_child= NULL;
+  optimizer::RangeReadPlan **cpk_scan= NULL;
   bool imerge_too_expensive= false;
   double imerge_cost= 0.0;
   ha_rows cpk_scan_records= 0;
@@ -1222,15 +1222,15 @@
   bool all_scans_ror_able= true;
   bool all_scans_rors= true;
   uint32_t unique_calc_buff_size;
-  optimizer::TABLE_READ_PLAN **roru_read_plans= NULL;
-  optimizer::TABLE_READ_PLAN **cur_roru_plan= NULL;
+  optimizer::TableReadPlan **roru_read_plans= NULL;
+  optimizer::TableReadPlan **cur_roru_plan= NULL;
   double roru_index_costs;
   ha_rows roru_total_records;
   double roru_intersect_part= 1.0;
 
-  if (!(range_scans= (optimizer::TRP_RANGE**)alloc_root(param->mem_root,
-                                                        sizeof(optimizer::TRP_RANGE*)*
-                                                        n_child_scans)))
+  if (! (range_scans= (optimizer::RangeReadPlan**)alloc_root(param->mem_root,
+                                                             sizeof(optimizer::RangeReadPlan*)*
+                                                             n_child_scans)))
     return NULL;
   /*
     Collect best 'range' scan for each of disjuncts, and, while doing so,
@@ -1280,7 +1280,7 @@
   }
   if (all_scans_rors)
   {
-    roru_read_plans= (optimizer::TABLE_READ_PLAN**)range_scans;
+    roru_read_plans= (optimizer::TableReadPlan **) range_scans;
     goto skip_to_ror_scan;
   }
   if (cpk_scan)
@@ -1323,7 +1323,7 @@
                          param->session->variables.sortbuff_size);
   if (imerge_cost < read_time)
   {
-    if ((imerge_trp= new (param->mem_root) optimizer::TRP_INDEX_MERGE))
+    if ((imerge_trp= new (param->mem_root) optimizer::IndexMergeReadPlan))
     {
       imerge_trp->read_cost= imerge_cost;
       imerge_trp->records= non_cpk_scan_records + cpk_scan_records;
@@ -1341,11 +1341,11 @@
 
   /* Ok, it is possible to build a ROR-union, try it. */
   bool dummy;
-  if (!(roru_read_plans=
-          (optimizer::TABLE_READ_PLAN**)alloc_root(param->mem_root,
-                                                   sizeof(optimizer::TABLE_READ_PLAN*)*
+  if (! (roru_read_plans=
+          (optimizer::TableReadPlan **) alloc_root(param->mem_root,
+                                                   sizeof(optimizer::TableReadPlan*)*
                                                    n_child_scans)))
-    return(imerge_trp);
+    return imerge_trp;
 skip_to_ror_scan:
   roru_index_costs= 0.0;
   roru_total_records= 0;
@@ -1374,7 +1374,7 @@
     else
       cost= read_time;
 
-    optimizer::TABLE_READ_PLAN *prev_plan= *cur_child;
+    optimizer::TableReadPlan *prev_plan= *cur_child;
     if (!(*cur_roru_plan= get_best_ror_intersect(param, *ptree, cost,
                                                  &dummy)))
     {
@@ -1386,7 +1386,7 @@
     }
     else
       roru_index_costs +=
-        ((optimizer::TRP_ROR_INTERSECT*)(*cur_roru_plan))->index_scan_costs;
+        ((optimizer::RorIntersectReadPlan*)(*cur_roru_plan))->index_scan_costs;
     roru_total_records += (*cur_roru_plan)->records;
     roru_intersect_part *= (*cur_roru_plan)->records /
                            param->table->cursor->stats.records;
@@ -1421,10 +1421,10 @@
                      sweep_cost.total_cost();
   }
 
-  optimizer::TRP_ROR_UNION *roru= NULL;
+  optimizer::RorUnionReadPlan *roru= NULL;
   if (roru_total_cost < read_time)
   {
-    if ((roru= new (param->mem_root) optimizer::TRP_ROR_UNION))
+    if ((roru= new (param->mem_root) optimizer::RorUnionReadPlan))
     {
       roru->first_ror= roru_read_plans;
       roru->last_ror= roru_read_plans + n_child_scans;
@@ -1954,10 +1954,10 @@
 */
 
 static
-optimizer::TRP_ROR_INTERSECT *get_best_ror_intersect(const optimizer::Parameter *param,
-                                                     SEL_TREE *tree,
-                                                     double read_time,
-                                                     bool *are_all_covering)
+optimizer::RorIntersectReadPlan *get_best_ror_intersect(const optimizer::Parameter *param,
+                                                   SEL_TREE *tree,
+                                                   double read_time,
+                                                   bool *are_all_covering)
 {
   uint32_t idx;
   double min_cost= DBL_MAX;
@@ -2080,10 +2080,10 @@
   }
 
   /* Ok, return ROR-intersect plan if we have found one */
-  optimizer::TRP_ROR_INTERSECT *trp= NULL;
+  optimizer::RorIntersectReadPlan *trp= NULL;
   if (min_cost < read_time && (cpk_scan_used || best_num > 1))
   {
-    if (! (trp= new (param->mem_root) optimizer::TRP_ROR_INTERSECT))
+    if (! (trp= new (param->mem_root) optimizer::RorIntersectReadPlan))
       return trp;
 
     if (! (trp->first_scan=
@@ -2141,9 +2141,9 @@
 */
 
 static
-optimizer::TRP_ROR_INTERSECT *get_best_covering_ror_intersect(optimizer::Parameter *param,
-                                                              SEL_TREE *tree,
-                                                              double read_time)
+optimizer::RorIntersectReadPlan *get_best_covering_ror_intersect(optimizer::Parameter *param,
+                                                            SEL_TREE *tree,
+                                                            double read_time)
 {
   ROR_SCAN_INFO **ror_scan_mark;
   ROR_SCAN_INFO **ror_scans_end= tree->ror_scans_end;
@@ -2233,8 +2233,8 @@
   if (total_cost > read_time)
     return NULL;
 
-  optimizer::TRP_ROR_INTERSECT *trp= NULL;
-  if (! (trp= new (param->mem_root) optimizer::TRP_ROR_INTERSECT))
+  optimizer::RorIntersectReadPlan *trp= NULL;
+  if (! (trp= new (param->mem_root) optimizer::RorIntersectReadPlan))
   {
     return trp;
   }
@@ -2282,11 +2282,11 @@
     NULL if no plan found or error occurred
 */
 
-static optimizer::TRP_RANGE *get_key_scans_params(optimizer::Parameter *param,
-                                                  SEL_TREE *tree,
-                                                  bool index_read_must_be_used,
-                                                  bool update_tbl_stats,
-                                                  double read_time)
+static optimizer::RangeReadPlan *get_key_scans_params(optimizer::Parameter *param,
+                                                      SEL_TREE *tree,
+                                                      bool index_read_must_be_used,
+                                                      bool update_tbl_stats,
+                                                      double read_time)
 {
   uint32_t idx;
   optimizer::SEL_ARG **key= NULL;
@@ -2295,7 +2295,7 @@
   ha_rows best_records= 0;
   uint32_t best_mrr_flags= 0;
   uint32_t best_buf_size= 0;
-  optimizer::TRP_RANGE *read_plan= NULL;
+  optimizer::RangeReadPlan *read_plan= NULL;
   /*
     Note that there may be trees that have type SEL_TREE::KEY but contain no
     key reads at all, e.g. tree for expression "key1 is not null" where key1
@@ -2344,8 +2344,8 @@
   if (key_to_read)
   {
     idx= key_to_read - tree->keys;
-    if ((read_plan= new (param->mem_root) optimizer::TRP_RANGE(*key_to_read, idx,
-                                                               best_mrr_flags)))
+    if ((read_plan= new (param->mem_root) optimizer::RangeReadPlan(*key_to_read, idx,
+                                                                   best_mrr_flags)))
     {
       read_plan->records= best_records;
       read_plan->is_ror= tree->ror_scans_map.test(idx);
@@ -2358,7 +2358,7 @@
 }
 
 
-optimizer::QuickSelectInterface *optimizer::TRP_INDEX_MERGE::make_quick(optimizer::Parameter *param, bool, memory::Root *)
+optimizer::QuickSelectInterface *optimizer::IndexMergeReadPlan::make_quick(optimizer::Parameter *param, bool, memory::Root *)
 {
   optimizer::QuickIndexMergeSelect *quick_imerge;
   optimizer::QuickRangeSelect *quick= NULL;
@@ -2370,7 +2370,7 @@
 
   quick_imerge->records= records;
   quick_imerge->read_time= read_cost;
-  for (optimizer::TRP_RANGE **range_scan= range_scans; 
+  for (optimizer::RangeReadPlan **range_scan= range_scans; 
        range_scan != range_scans_end;
        range_scan++)
   {
@@ -2386,9 +2386,9 @@
   return quick_imerge;
 }
 
-optimizer::QuickSelectInterface *optimizer::TRP_ROR_INTERSECT::make_quick(optimizer::Parameter *param,
-                                                                          bool retrieve_full_rows,
-                                                                          memory::Root *parent_alloc)
+optimizer::QuickSelectInterface *optimizer::RorIntersectReadPlan::make_quick(optimizer::Parameter *param,
+                                                                             bool retrieve_full_rows,
+                                                                             memory::Root *parent_alloc)
 {
   optimizer::QuickRorIntersectSelect *quick_intersect= NULL;
   optimizer::QuickRangeSelect *quick= NULL;
@@ -2441,10 +2441,10 @@
 }
 
 
-optimizer::QuickSelectInterface *optimizer::TRP_ROR_UNION::make_quick(optimizer::Parameter *param, bool, memory::Root *)
+optimizer::QuickSelectInterface *optimizer::RorUnionReadPlan::make_quick(optimizer::Parameter *param, bool, memory::Root *)
 {
   optimizer::QuickRorUnionSelect *quick_roru= NULL;
-  optimizer::TABLE_READ_PLAN **scan= NULL;
+  optimizer::TableReadPlan **scan= NULL;
   optimizer::QuickSelectInterface *quick= NULL;
   /*
     It is impossible to construct a ROR-union that will not retrieve full
@@ -5350,13 +5350,13 @@
 
   RETURN
     If mem_root != NULL
-    - valid TRP_GROUP_MIN_MAX object if this QUICK class can be used for
+    - valid GroupMinMaxReadPlan object if this QUICK class can be used for
       the query
     -  NULL o/w.
     If mem_root == NULL
     - NULL
 */
-static optimizer::TRP_GROUP_MIN_MAX *
+static optimizer::GroupMinMaxReadPlan *
 get_best_group_min_max(optimizer::Parameter *param, SEL_TREE *tree)
 {
   Session *session= param->session;
@@ -5373,7 +5373,7 @@
   uint32_t used_key_parts= 0;   /* Number of index key parts used for access. */
   unsigned char key_infix[MAX_KEY_LENGTH]; /* Constants from equality predicates.*/
   uint32_t key_infix_len= 0;          /* Length of key_infix. */
-  optimizer::TRP_GROUP_MIN_MAX *read_plan= NULL; /* The eventually constructed TRP. */
+  optimizer::GroupMinMaxReadPlan *read_plan= NULL; /* The eventually constructed TRP. */
   uint32_t key_part_nr;
   order_st *tmp_group= NULL;
   Item *item= NULL;
@@ -5772,20 +5772,20 @@
 
   /* The query passes all tests, so construct a new TRP object. */
   read_plan=
-    new(param->mem_root) optimizer::TRP_GROUP_MIN_MAX(have_min,
-                                                      have_max,
-                                                      min_max_arg_part,
-                                                      group_prefix_len,
-                                                      used_key_parts,
-                                                      group_key_parts,
-                                                      index_info,
-                                                      index,
-                                                      key_infix_len,
-                                                      (key_infix_len > 0) ? key_infix : NULL,
-                                                      tree,
-                                                      best_index_tree,
-                                                      best_param_idx,
-                                                      best_quick_prefix_records);
+    new(param->mem_root) optimizer::GroupMinMaxReadPlan(have_min,
+                                                        have_max,
+                                                        min_max_arg_part,
+                                                        group_prefix_len,
+                                                        used_key_parts,
+                                                        group_key_parts,
+                                                        index_info,
+                                                        index,
+                                                        key_infix_len,
+                                                        (key_infix_len > 0) ? key_infix : NULL,
+                                                        tree,
+                                                        best_index_tree,
+                                                        best_param_idx,
+                                                        best_quick_prefix_records);
   if (read_plan)
   {
     if (tree && read_plan->quick_prefix_records == 0)
@@ -6117,7 +6117,7 @@
     records             [out] The number of rows retrieved
 
   DESCRIPTION
-    This method computes the access cost of a TRP_GROUP_MIN_MAX instance and
+    This method computes the access cost of a GroupMinMaxReadPlan instance and
     the number of rows returned. It updates this->read_cost and this->records.
 
   NOTES
@@ -6241,7 +6241,7 @@
   Construct a new quick select object for queries with group by with min/max.
 
   SYNOPSIS
-    TRP_GROUP_MIN_MAX::make_quick()
+    GroupMinMaxReadPlan::make_quick()
     param              Parameter from test_quick_select
     retrieve_full_rows ignored
     parent_alloc       Memory pool to use, if any.
@@ -6258,7 +6258,7 @@
     NULL otherwise.
 */
 optimizer::QuickSelectInterface *
-optimizer::TRP_GROUP_MIN_MAX::make_quick(optimizer::Parameter *param, bool, memory::Root *parent_alloc)
+optimizer::GroupMinMaxReadPlan::make_quick(optimizer::Parameter *param, bool, memory::Root *parent_alloc)
 {
   optimizer::QuickGroupMinMaxSelect *quick= NULL;
 
@@ -6346,7 +6346,7 @@
 }
 
 
-optimizer::QuickSelectInterface *optimizer::TRP_RANGE::make_quick(optimizer::Parameter *param, bool, memory::Root *parent_alloc)
+optimizer::QuickSelectInterface *optimizer::RangeReadPlan::make_quick(optimizer::Parameter *param, bool, memory::Root *parent_alloc)
 {
   optimizer::QuickRangeSelect *quick= NULL;
   if ((quick= optimizer::get_quick_select(param,

=== modified file 'drizzled/optimizer/range.h'
--- drizzled/optimizer/range.h	2009-12-25 09:11:55 +0000
+++ drizzled/optimizer/range.h	2010-01-26 19:00:26 +0000
@@ -28,7 +28,7 @@
 #include <queue>
 
 class JOIN;
-class TRP_ROR_INTERSECT; 
+class RorIntersectReadPlan; 
 typedef class Item COND;
 typedef struct st_io_cache IO_CACHE;
 

=== modified file 'drizzled/optimizer/range_param.h'
--- drizzled/optimizer/range_param.h	2009-12-25 08:50:15 +0000
+++ drizzled/optimizer/range_param.h	2010-01-26 19:12:51 +0000
@@ -34,6 +34,26 @@
 class RangeParameter
 {
 public:
+
+  RangeParameter()
+    :
+      session(NULL),
+      table(NULL),
+      cond(NULL),
+      prev_tables(),
+      read_tables(),
+      current_table(),
+      key_parts(NULL),
+      key_parts_end(NULL),
+      mem_root(NULL),
+      old_root(NULL),
+      keys(0),
+      using_real_indexes(false),
+      remove_jump_scans(false),
+      alloced_sel_args(0),
+      force_default_mrr(false)
+  {}
+
   Session	*session;   /* Current thread handle */
   Table *table; /* Table being analyzed */
   COND *cond;   /* Used inside get_mm_tree(). */
@@ -74,13 +94,30 @@
 class Parameter : public RangeParameter
 {
 public:
+
+  Parameter()
+    :
+      RangeParameter(),
+      max_key_part(0),
+      range_count(0),
+      quick(false),
+      fields_bitmap_size(0),
+      needed_fields(),
+      tmp_covered_fields(),
+      needed_reg(NULL),
+      imerge_cost_buff(NULL),
+      imerge_cost_buff_size(0),
+      is_ror_scan(false),
+      n_ranges(0)
+  {}
+
   KEY_PART *key[MAX_KEY]; /* First key parts of keys used in the query */
   uint32_t max_key_part;
   /* Number of ranges in the last checked tree->key */
   uint32_t range_count;
-  unsigned char min_key[MAX_KEY_LENGTH+MAX_FIELD_WIDTH],
-    max_key[MAX_KEY_LENGTH+MAX_FIELD_WIDTH];
-  bool quick;				// Don't calulate possible keys
+  unsigned char min_key[MAX_KEY_LENGTH+MAX_FIELD_WIDTH];
+  unsigned char max_key[MAX_KEY_LENGTH+MAX_FIELD_WIDTH];
+  bool quick; // Don't calulate possible keys
 
   uint32_t fields_bitmap_size;
   MyBitmap needed_fields;    /* bitmask of fields needed by the query */

=== modified file 'drizzled/optimizer/table_read_plan.h'
--- drizzled/optimizer/table_read_plan.h	2009-12-25 08:50:15 +0000
+++ drizzled/optimizer/table_read_plan.h	2010-01-27 17:32:16 +0000
@@ -36,7 +36,7 @@
   Table rows retrieval plan. Range optimizer creates QuickSelectInterface-derived
   objects from table read plans.
 */
-class TABLE_READ_PLAN
+class TableReadPlan
 {
 public:
   /*
@@ -84,18 +84,18 @@
   static void operator delete(void *, drizzled::memory::Root *)
     { /* Never called */ }
 
-  virtual ~TABLE_READ_PLAN() {} /* Remove gcc warning */
+  virtual ~TableReadPlan() {} /* Remove gcc warning */
 
 };
 
 
 /*
   Plan for a QuickRangeSelect scan.
-  TRP_RANGE::make_quick ignores retrieve_full_rows parameter because
+  RangeReadPlan::make_quick ignores retrieve_full_rows parameter because
   QuickRangeSelect doesn't distinguish between 'index only' scans and full
   record retrieval scans.
 */
-class TRP_RANGE : public TABLE_READ_PLAN
+class RangeReadPlan : public TableReadPlan
 {
 
 public:
@@ -105,13 +105,13 @@
   uint32_t     mrr_flags;
   uint32_t     mrr_buf_size;
 
-  TRP_RANGE(SEL_ARG *key_arg, uint32_t idx_arg, uint32_t mrr_flags_arg)
+  RangeReadPlan(SEL_ARG *key_arg, uint32_t idx_arg, uint32_t mrr_flags_arg)
     :
       key(key_arg),
       key_idx(idx_arg),
       mrr_flags(mrr_flags_arg)
   {}
-  virtual ~TRP_RANGE() {}                     /* Remove gcc warning */
+  virtual ~RangeReadPlan() {}                     /* Remove gcc warning */
 
   QuickSelectInterface *make_quick(Parameter *param, bool, drizzled::memory::Root *parent_alloc);
 
@@ -119,12 +119,11 @@
 
 
 /* Plan for QuickRorIntersectSelect scan. */
-
-class TRP_ROR_INTERSECT : public TABLE_READ_PLAN
+class RorIntersectReadPlan : public TableReadPlan
 {
 public:
-  TRP_ROR_INTERSECT() {}                      /* Remove gcc warning */
-  virtual ~TRP_ROR_INTERSECT() {}             /* Remove gcc warning */
+  RorIntersectReadPlan() {}                      /* Remove gcc warning */
+  virtual ~RorIntersectReadPlan() {}             /* Remove gcc warning */
   QuickSelectInterface *make_quick(Parameter *param,
                                    bool retrieve_full_rows,
                                    drizzled::memory::Root *parent_alloc);
@@ -144,16 +143,16 @@
   is ignored by make_quick.
 */
 
-class TRP_ROR_UNION : public TABLE_READ_PLAN
+class RorUnionReadPlan : public TableReadPlan
 {
 public:
-  TRP_ROR_UNION() {}                          /* Remove gcc warning */
-  virtual ~TRP_ROR_UNION() {}                 /* Remove gcc warning */
+  RorUnionReadPlan() {}                          /* Remove gcc warning */
+  virtual ~RorUnionReadPlan() {}                 /* Remove gcc warning */
   QuickSelectInterface *make_quick(Parameter *param,
                                    bool retrieve_full_rows,
                                    drizzled::memory::Root *parent_alloc);
-  TABLE_READ_PLAN **first_ror; /* array of ptrs to plans for merged scans */
-  TABLE_READ_PLAN **last_ror;  /* end of the above array */
+  TableReadPlan **first_ror; /* array of ptrs to plans for merged scans */
+  TableReadPlan **last_ror;  /* end of the above array */
 };
 
 
@@ -163,16 +162,16 @@
   is ignored by make_quick.
 */
 
-class TRP_INDEX_MERGE : public TABLE_READ_PLAN
+class IndexMergeReadPlan : public TableReadPlan
 {
 public:
-  TRP_INDEX_MERGE() {}                        /* Remove gcc warning */
-  virtual ~TRP_INDEX_MERGE() {}               /* Remove gcc warning */
+  IndexMergeReadPlan() {}                        /* Remove gcc warning */
+  virtual ~IndexMergeReadPlan() {}               /* Remove gcc warning */
   QuickSelectInterface *make_quick(Parameter *param,
                                    bool retrieve_full_rows,
                                    drizzled::memory::Root *parent_alloc);
-  TRP_RANGE **range_scans; /* array of ptrs to plans of merged scans */
-  TRP_RANGE **range_scans_end; /* end of the array */
+  RangeReadPlan **range_scans; /* array of ptrs to plans of merged scans */
+  RangeReadPlan **range_scans_end; /* end of the array */
 };
 
 
@@ -180,7 +179,7 @@
   Plan for a QuickGroupMinMaxSelect scan.
 */
 
-class TRP_GROUP_MIN_MAX : public TABLE_READ_PLAN
+class GroupMinMaxReadPlan : public TableReadPlan
 {
 private:
   bool have_min;
@@ -201,20 +200,20 @@
   ha_rows quick_prefix_records;
 
 public:
-  TRP_GROUP_MIN_MAX(bool have_min_arg, 
-                    bool have_max_arg,
-                    KEY_PART_INFO *min_max_arg_part_arg,
-                    uint32_t group_prefix_len_arg, 
-                    uint32_t used_key_parts_arg,
-                    uint32_t group_key_parts_arg, 
-                    KEY *index_info_arg,
-                    uint32_t index_arg, 
-                    uint32_t key_infix_len_arg,
-                    unsigned char *key_infix_arg,
-                    SEL_TREE *tree_arg, 
-                    SEL_ARG *index_tree_arg,
-                    uint32_t param_idx_arg, 
-                    ha_rows quick_prefix_records_arg)
+  GroupMinMaxReadPlan(bool have_min_arg, 
+                      bool have_max_arg,
+                      KEY_PART_INFO *min_max_arg_part_arg,
+                      uint32_t group_prefix_len_arg, 
+                      uint32_t used_key_parts_arg,
+                      uint32_t group_key_parts_arg, 
+                      KEY *index_info_arg,
+                      uint32_t index_arg, 
+                      uint32_t key_infix_len_arg,
+                      unsigned char *key_infix_arg,
+                      SEL_TREE *tree_arg, 
+                      SEL_ARG *index_tree_arg,
+                      uint32_t param_idx_arg, 
+                      ha_rows quick_prefix_records_arg)
     :
       have_min(have_min_arg),
       have_max(have_max_arg),
@@ -233,7 +232,7 @@
       if (key_infix_len)
         memcpy(this->key_infix, key_infix_arg, key_infix_len);
     }
-  virtual ~TRP_GROUP_MIN_MAX() {}             /* Remove gcc warning */
+  virtual ~GroupMinMaxReadPlan() {}             /* Remove gcc warning */
 
   QuickSelectInterface *make_quick(Parameter *param,
                                    bool retrieve_full_rows,

=== modified file 'drizzled/sql_lex.h'
--- drizzled/sql_lex.h	2009-12-27 00:49:03 +0000
+++ drizzled/sql_lex.h	2010-01-27 04:37:14 +0000
@@ -36,6 +36,7 @@
 #include "drizzled/key_part_spec.h"
 #include "drizzled/index_hint.h"
 #include "drizzled/statement.h"
+#include "drizzled/optimizer/explain_plan.h"
 
 #include <bitset>
 #include <string>
@@ -424,7 +425,7 @@
     by TableList::next_leaf, so leaf_tables points to the left-most leaf.
   */
   TableList *leaf_tables;
-  std::string type; /* type of select for EXPLAIN          */
+  enum drizzled::optimizer::select_type type; /* type of select for EXPLAIN */
 
   SQL_LIST order_list;                /* ORDER clause */
   SQL_LIST *gorder_list;

